@@ -17,18 +17,28 @@ test_etcd3-gateway
 Tests for `etcd3gw` module.
 """
 
+import base64
+import json
+import requests
 import six
 import threading
 import time
 import uuid
 
 from testtools.testcase import unittest
+from unittest import mock
 import urllib3
 
 from etcd3gw.client import Etcd3Client
 from etcd3gw import exceptions
 from etcd3gw.tests import base
 
+try:
+    # Python 3.8 : time.clock was deprecated and removed.
+    from time import perf_counter as clock
+except ImportError:
+    from time import clock
+
 
 def _is_etcd3_running():
     try:
@@ -344,7 +354,7 @@ class TestEtcd3Gateway(base.TestCase):
     @unittest.skipUnless(
         _is_etcd3_running(), "etcd3 is not available")
     def test_client_locks(self):
-        lock = self.client.lock(id='xyz-%s' % time.clock(), ttl=60)
+        lock = self.client.lock(id='xyz-%s' % clock(), ttl=60)
         self.assertIsNotNone(lock)
 
         self.assertTrue(lock.acquire())
@@ -382,3 +392,46 @@ class TestEtcd3Gateway(base.TestCase):
         # Verify that key is still 'bar'
         self.assertEqual([six.b('bar')], self.client.get(key))
         self.assertFalse(status)
+
+    @unittest.skipUnless(
+        _is_etcd3_running(), "etcd3 is not available")
+    def test_create_with_lease_success(self):
+        key = '/foo/unique' + str(uuid.uuid4())
+        # Verify that key is empty
+        self.assertEqual([], self.client.get(key))
+        lease = self.client.lease()
+
+        status = self.client.create(key, 'bar', lease=lease)
+        # Verify that key is 'bar'
+        self.assertEqual([six.b('bar')], self.client.get(key))
+        self.assertTrue(status)
+        keys = lease.keys()
+        self.assertEqual(1, len(keys))
+        self.assertIn(six.b(key), keys)
+
+    def my_iter_content(self, *args, **kwargs):
+        payload = json.dumps({
+            'result': {
+                'events': [{
+                    'kv': {'key': base64.b64encode(b'value').decode('utf-8')},
+                }]
+            }
+        })
+
+        if not kwargs.get('decode_unicode', False):
+            payload = payload.encode()
+        return [payload]
+
+    @mock.patch.object(requests.Response, 'iter_content', new=my_iter_content)
+    @mock.patch.object(requests.sessions.Session, 'post')
+    def test_watch_unicode(self, mock_post):
+        mocked_response = requests.Response()
+        mocked_response.connection = mock.Mock()
+        mock_post.return_value = mocked_response
+
+        try:
+            res = self.client.watch_once('/some/key', timeout=1)
+        except exceptions.WatchTimedOut:
+            self.fail("watch timed out when server responded with unicode")
+
+        self.assertEqual(res, {'kv': {'key': b'value'}})
